@@ -1,24 +1,9 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2011-2014  Intel Corporation
+ * Copyright (C) 2017  Codecoup
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2011-2014  Intel Corporation. All rights reserved.
- *  Copyright (C) 2017  Codecoup. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -609,17 +594,18 @@ LIB_EXPORT struct l_dbus_client *l_dbus_client_new(struct l_dbus *dbus,
 
 	client->dbus = dbus;
 
+	client->service = l_strdup(service);
 	client->watch = l_dbus_add_service_watch(dbus, service,
 						service_appeared_callback,
 						service_disappeared_callback,
 						client, NULL);
 
 	if (!client->watch) {
+		l_free(client->service);
 		l_free(client);
 		return NULL;
 	}
 
-	client->service = l_strdup(service);
 	client->proxies = l_queue_new();
 
 	return client;
@@ -631,7 +617,7 @@ LIB_EXPORT void l_dbus_client_destroy(struct l_dbus_client *client)
 		return;
 
 	if (client->watch)
-		l_dbus_remove_signal_watch(client->dbus, client->watch);
+		l_dbus_remove_watch(client->dbus, client->watch);
 
 	if (client->added_watch)
 		l_dbus_remove_signal_watch(client->dbus, client->added_watch);
