@@ -1,24 +1,14 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2020  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2020  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdint.h>
 #include <linux/types.h>
 #include <errno.h>
@@ -29,6 +19,7 @@
 
 #include "dhcp-private.h"
 #include "private.h"
+#include "useful.h"
 
 #define BITS_PER_LONG (sizeof(unsigned long) * 8)
 
@@ -72,7 +63,9 @@ bool _dhcp_message_builder_append(struct dhcp_message_builder *builder,
 
 		builder->pos[0] = code;
 		builder->pos[1] = optlen;
-		memcpy(builder->pos + 2, optval, optlen);
+
+		if (optlen)
+			memcpy(builder->pos + 2, optval, optlen);
 
 		builder->pos += optlen + 2;
 		break;
