@@ -1,23 +1,8 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2011-2014  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2011-2014  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,16 +13,21 @@
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <limits.h>
 #include <signal.h>
 #include <sys/epoll.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 
 #include "signal.h"
 #include "queue.h"
 #include "log.h"
-#include "util.h"
+#include "useful.h"
 #include "main.h"
+#include "main-private.h"
 #include "private.h"
+#include "timeout.h"
 
 /**
  * SECTION:main
@@ -54,11 +44,17 @@
 #define WATCH_FLAG_DISPATCHING	1
 #define WATCH_FLAG_DESTROYED	2
 
-static int epoll_fd;
+#define WATCHDOG_TRIGGER_FREQ	2
+
+static int epoll_fd = -1;
 static bool epoll_running;
 static bool epoll_terminate;
 static int idle_id;
 
+static int notify_fd;
+
+static struct l_timeout *watchdog;
+
 static struct l_queue *idle_list;
 
 struct watch_data {
@@ -88,10 +84,8 @@ static inline bool __attribute__ ((always_inline)) create_epoll(void)
 	unsigned int i;
 
 	epoll_fd = epoll_create1(EPOLL_CLOEXEC);
-	if (epoll_fd < 0) {
-		epoll_fd = 0;
+	if (epoll_fd < 0)
 		return false;
-	}
 
 	watch_list = malloc(DEFAULT_WATCH_ENTRIES * sizeof(void *));
 	if (!watch_list)
@@ -110,7 +104,7 @@ static inline bool __attribute__ ((always_inline)) create_epoll(void)
 
 close_epoll:
 	close(epoll_fd);
-	epoll_fd = 0;
+	epoll_fd = -1;
 
 	return false;
 }
@@ -125,10 +119,10 @@ int watch_add(int fd, uint32_t events, watch_event_cb_t callback,
 	if (unlikely(fd < 0 || !callback))
 		return -EINVAL;
 
-	if (!epoll_fd)
+	if (epoll_fd < 0)
 		return -EIO;
 
-	if ((unsigned int) fd > watch_entries - 1)
+	if (L_WARN_ON((unsigned int) fd > watch_entries - 1))
 		return -ERANGE;
 
 	data = l_new(struct watch_data, 1);
@@ -214,17 +208,21 @@ int watch_clear(int fd)
 	return 0;
 }
 
-int watch_remove(int fd)
+int watch_remove(int fd, bool epoll_del)
 {
 	int err = watch_clear(fd);
 
 	if (err < 0)
 		return err;
 
+	if (!epoll_del)
+		goto done;
+
 	err = epoll_ctl(epoll_fd, EPOLL_CTL_DEL, fd, NULL);
 	if (err < 0)
 		return -errno;
 
+done:
 	return err;
 }
 
@@ -269,7 +267,7 @@ int idle_add(idle_event_cb_t callback, void *user_data, uint32_t flags,
 	if (unlikely(!callback))
 		return -EINVAL;
 
-	if (!epoll_fd)
+	if (epoll_fd < 0)
 		return -EIO;
 
 	data = l_new(struct idle_data, 1);
@@ -324,6 +322,80 @@ static void idle_dispatch(void *data, void *user_data)
 	idle->flags &= ~IDLE_FLAG_DISPATCHING;
 }
 
+static int sd_notify(const char *state)
+{
+	int err;
+
+	if (notify_fd <= 0)
+		return -ENOTCONN;
+
+	err = send(notify_fd, state, strlen(state), MSG_NOSIGNAL);
+	if (err < 0)
+		return -errno;
+
+	return 0;
+}
+
+static void watchdog_callback(struct l_timeout *timeout, void *user_data)
+{
+	int msec = L_PTR_TO_INT(user_data);
+
+	sd_notify("WATCHDOG=1");
+
+	l_timeout_modify_ms(timeout, msec);
+}
+
+static void create_sd_notify_socket(void)
+{
+	const char *sock;
+	struct sockaddr_un addr;
+	const char *watchdog_usec;
+	socklen_t len;
+	int msec;
+
+	/* check if NOTIFY_SOCKET has been set */
+	sock = getenv("NOTIFY_SOCKET");
+	if (!sock)
+		return;
+
+	/* check for abstract socket or absolute path */
+	if (sock[0] != '@' && sock[0] != '/')
+		return;
+
+	notify_fd = socket(AF_UNIX, SOCK_DGRAM | SOCK_CLOEXEC, 0);
+	if (notify_fd < 0) {
+		notify_fd = 0;
+		return;
+	}
+
+	memset(&addr, 0, sizeof(addr));
+	addr.sun_family = AF_UNIX;
+	strncpy(addr.sun_path, sock, sizeof(addr.sun_path) - 1);
+	len = offsetof(struct sockaddr_un, sun_path) + strlen(sock);
+
+	if (addr.sun_path[0] == '@')
+		addr.sun_path[0] = '\0';
+
+	if (connect(notify_fd, (struct sockaddr *) &addr, len) < 0) {
+		close(notify_fd);
+		notify_fd = 0;
+		return;
+	}
+
+	watchdog_usec = getenv("WATCHDOG_USEC");
+	if (!watchdog_usec)
+		return;
+
+	msec = atoi(watchdog_usec) / 1000;
+	if (msec < WATCHDOG_TRIGGER_FREQ)
+		return;
+
+	msec /= WATCHDOG_TRIGGER_FREQ;
+
+	watchdog = l_timeout_create_ms(msec, watchdog_callback,
+					L_INT_TO_PTR(msec), NULL);
+}
+
 /**
  * l_main_init:
  *
@@ -342,6 +414,8 @@ LIB_EXPORT bool l_main_init(void)
 	if (!create_epoll())
 		return false;
 
+	create_sd_notify_socket();
+
 	epoll_terminate = false;
 
 	return true;
@@ -420,12 +494,13 @@ LIB_EXPORT int l_main_run(void)
 	int timeout;
 
 	/* Has l_main_init() been called? */
-	if (unlikely(!epoll_fd))
+	if (unlikely(epoll_fd < 0))
 		return EXIT_FAILURE;
 
 	if (unlikely(epoll_running))
 		return EXIT_FAILURE;
 
+	sd_notify("READY=1");
 	epoll_running = true;
 
 	for (;;) {
@@ -438,6 +513,13 @@ LIB_EXPORT int l_main_run(void)
 
 	epoll_running = false;
 
+	if (notify_fd) {
+		close(notify_fd);
+		notify_fd = 0;
+		l_timeout_remove(watchdog);
+		watchdog = NULL;
+	}
+
 	return EXIT_SUCCESS;
 }
 
@@ -481,7 +563,7 @@ LIB_EXPORT bool l_main_exit(void)
 	idle_list = NULL;
 
 	close(epoll_fd);
-	epoll_fd = 0;
+	epoll_fd = -1;
 
 	return true;
 }
@@ -489,7 +571,7 @@ LIB_EXPORT bool l_main_exit(void)
 /**
  * l_main_quit:
  *
- * Teminate the running main loop
+ * Terminate the running main loop
  *
  * Returns: #true when terminating the main loop or #false in case of failure
  **/
