@@ -1,23 +1,8 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2015  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2015  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,8 +14,9 @@
 #include <unistd.h>
 #include <sys/mman.h>
 #include <strings.h>
+#include <errno.h>
 
-#include "util.h"
+#include "useful.h"
 #include "private.h"
 #include "key.h"
 #include "cert.h"
@@ -162,7 +148,7 @@ const char *pem_next(const void *buf, size_t buf_len, char **type_label,
 				*base64_len = buf_ptr - base64_data;
 
 			if (endp) {
-				if (eol == buf + buf_len)
+				if (eol == buf_ptr + buf_len)
 					*endp = eol;
 				else
 					*endp = eol + 1;
@@ -378,7 +364,8 @@ static bool pem_write_one_cert(struct l_cert *cert, void *user_data)
 
 	iov[0].iov_base = "-----BEGIN CERTIFICATE-----\n";
 	iov[0].iov_len = strlen(iov[0].iov_base);
-	iov[1].iov_base = l_base64_encode(der, der_len, 64, &iov[1].iov_len);
+	iov[1].iov_base = l_base64_encode(der, der_len, 64);
+	iov[1].iov_len = strlen(iov[1].iov_base);
 	iov[2].iov_base = "\n-----END CERTIFICATE-----\n";
 	iov[2].iov_len = strlen(iov[2].iov_base);
 	r = L_TFR(writev(*fd, iov, 3));
