@@ -543,10 +543,10 @@ static void test_aead(const void *data)
 	success = l_aead_cipher_encrypt(cipher, pt, ptlen, aad, aadlen,
 					nonce, noncelen, encbuf, encbuflen);
 	if (!success) {
-		printf("* Some kernel versions before v4.9 have a known AEAD\n"
-			"* bug. If the system running this test is using a\n"
-			"* v4.8 or earlier kernel, a failure here is likely\n"
-			"* due to that kernel bug.\n");
+		printf("# Some kernel versions before v4.9 have a known AEAD\n"
+			"# bug. If the system running this test is using a\n"
+			"# v4.8 or earlier kernel, a failure here is likely\n"
+			"# due to that kernel bug.\n");
 	}
 	assert(success);
 
